#!/bin/csh -f
#
# Usage: compile.ibm mc-tinker_path
#
# C shell script used to compile MC-TINKER source code on SGI
#
if ($#argv != 1) then
   echo "Usage: compile.ibm mc-tinker_path"
   echo "For example: compile.ibm /usr/mc-tinker2010-1"
   exit 1
endif
set mctdir = $argv[1]
set tinkerdir = $mctdir/tinker/source
set wrkdir = $mctdir/wrk
set exedir = $mctdir/exe
set srcdir = $mctdir/srcmct
#
mkdir $wrkdir
cd $wrkdir
#
set CF = "g77 -c"
#set CF = "ifort -c -static -inline-debug-info -save"
#set CF = "efc -c -static -w"
#
# Copy some include files
#
cp $srcdir/sizes.i sizes.i
cp $srcdir/mcparm.i mcparm.i
cp $srcdir/mccomm.inc mccomm.inc
cp $srcdir/mcperc.inc mcperc.inc
cp $srcdir/mcparm.inc mcparm.inc
cp $srcdir/eemcmm.inc eemcmm.inc
#
# Copy tinker subroutines
#
cd $tinkerdir
#
cat getxyz.f mechanic.f nextarg.f upcase.f trimtext.f analysis.f \
gyrate.f inertia.f version.f prtxyz.f tncg.f gradient.f hessian.f \
writeout.f energy.f numgrad.f freeunit.f promo.f precise.f command.f \
basefile.f suffix.f readxyz.f attach.f active.f cluster.f bonds.f \
angles.f torsions.f cutoffs.f rings.f katom.f orbital.f kbond.f \
kangle.f kstrbnd.f kurey.f kangang.f kopbend.f kimprop.f kimptor.f \
ktors.f kstrtor.f kvdw.f kcharge.f kdipole.f kmpole.f kpolar.f solvate.f \
korbit.f molecule.f unitcell.f lattice.f restrain.f smooth.f mutate.f \
fatal.f bounds.f ebond3.f eangle3.f estrbnd3.f eurey3.f eangang3.f \
eopbend3.f eimprop3.f eimptor3.f etors3.f estrtor3.f ebuck3.f elj3.f ehal3.f \
egauss3.f echarge9.f echarge3.f echgdpl3.f edipole3.f empole3.f erxnfld3.f  \
esolv3.f egeom3.f extra3.f jacobi.f lowcase.f gettext.f born.f solve.f \
search.f ebond1.f eangle1.f estrbnd1.f eurey1.f eangang1.f \
eopbend1.f eimprop1.f ebuck2.f elj2.f erxnfld2.f precond.f lights.f empole1.f \
eimptor1.f etors1.f estrtor1.f ebuck5.f elj5.f ehal5.f ebuck1.f elj1.f \
ehal1.f egauss1.f echarge7.f echarge5.f echarge1.f echgdpl1.f edipole1.f \
erxnfld1.f esolv1.f egeom1.f extra1.f ebond2.f eangle2.f estrbnd2.f eurey2.f \
eangang2.f eopbend2.f eimprop2.f eimptor2.f etors2.f estrtor2.f \
ehal2.f egauss2.f echarge8.f echarge2.f echgdpl2.f edipole2.f empole2.f \
esolv2.f egeom2.f extra2.f numeral.f prtint.f ebond.f eangle.f estrbnd.f \
eurey.f eangang.f eopbend.f eimprop.f eimptor.f etors.f estrtor.f ebuck4.f \
elj4.f ehal4.f ebuck.f elj.f ehal.f egauss.f echarge6.f echarge4.f echarge.f \
echgdpl.f edipole.f empole.f erxnfld.f esolv.f egeom.f extra.f \
control.f nexttext.f getword.f sort.f getnumb.f \
prmkey.f getstring.f piplane.f number.f \
torphase.f overlap.f pitilt.f pialter.f groups.f switch.f \
image.f rotpole.f induce.f surface.f surfatom.f \
readprm.f pimove.f prterr.f cholesky.f column.f > $wrkdir/tinker.f
#
mv sizes.i sizes.i.bak
cp *.i $wrkdir
mv sizes.i.bak sizes.i
#
cd $wrkdir
#
# Copy MC-TINKER files
#
cp $srcdir/main.f main.f
cp $srcdir/mcsrc1.f mcsrc1.f
cp $srcdir/mcsrc2.f mcsrc2.f
cp $srcdir/mcsrc3.f mcsrc3.f
cp $srcdir/mcsrc4.f mcsrc4.f
cp $srcdir/mcsrc5.f mcsrc5.f
cp $srcdir/mcsrc6.f mcsrc6.f
cp $srcdir/mcsrc7.f mcsrc7.f
cp $srcdir/mcsrc8.f mcsrc8.f
cp $srcdir/mcfset.f mcfset.f
cp $srcdir/mcrsmldum.f mcrsmldum.f
cp $srcdir/emsrc1.f emsrc1.f 
cp $srcdir/getkey.f getkey.f
cp $srcdir/getprm.f getprm.f
cp $srcdir/energ.f energ.f
cp $srcdir/grads.f grads.f
cp $srcdir/hesss.f hesss.f
cp $srcdir/field.f field.f
cp $srcdir/piscf.f piscf.f
cp $srcdir/initial.f initial.f
cp $srcdir/readfile.f readfile.f
#
# Copy machine-dependent MC-TINKER files
#
cp $srcdir/mcdatm.linux2 mcdatm.f
#
# Compile MC-TINKER files
#
echo "======================== main ==========================="
$CF main.f
echo "======================== mcsrc1 ==========================="
$CF mcsrc1.f
echo "======================== mcsrc2 ==========================="
$CF mcsrc2.f
echo "======================== mcsrc3 ==========================="
$CF mcsrc3.f
echo "======================== mcsrc4 ==========================="
$CF mcsrc4.f
echo "======================== mcsrc5 ==========================="
$CF mcsrc5.f
echo "======================== mcsrc6 ==========================="
$CF mcsrc6.f
echo "======================== mcsrc7 ==========================="
$CF mcsrc7.f
echo "======================== mcsrc8 ==========================="
$CF mcsrc8.f
echo "======================== mcfset ==========================="
$CF mcfset.f
echo "======================== mcrsmldum ==========================="
$CF mcrsmldum.f
echo "======================== emsrc1 ==========================="
$CF emsrc1.f
echo "======================== getkey ==========================="
$CF getkey.f
echo "======================== getprm ==========================="
$CF getprm.f
echo "======================== energ ============================="
$CF energ.f
echo "======================== grads ============================="
$CF grads.f
echo "======================== hesss ============================="
$CF hesss.f
echo "======================== field ============================="
$CF field.f
echo "======================== piscf ============================="
$CF piscf.f
echo "======================== initial ==========================="
$CF initial.f
echo "======================== readfile =========================="
$CF readfile.f
echo "======================== mcdatm ==========================="
$CF mcdatm.f
#
# Compile tinker files
#
echo "======================== tinker ============================"
$CF tinker.f
#
# Link
#
g77 -o $exedir/mc-tinker20082.exe tinker.o main.o mcsrc1.o mcsrc2.o \
mcsrc3.o mcsrc4.o mcsrc5.o mcsrc6.o mcsrc7.o mcsrc8.o mcfset.o mcrsmldum.o emsrc1.o\
getkey.o getprm.o energ.o grads.o hesss.o field.o piscf.o initial.o \
readfile.o mcdatm.o
#
cd ..
#
# Comment this line if you dont want the termporary files to be deleted
#
/bin/rm -r $wrkdir
#

