c#######################################################################
c
c     PARAMETER STATEMENTS FOR MC-TINKER
c
c#######################################################################
c
c     This file contains the parameters that can be modified by the user.
c     Specifically, the parameters are:
c
c     mcmatm = maximum allowed number of atoms in the molecular system
c              (the MC-TINKER echivalent of the TINKER maxatm).
c
c     mcmkey = maximum allowed number of lines in the keyword file
c              (the MC-TINKER echivalent of the TINKER maxkey). For
c              simplicity mcmkey is set to twice the value of mcmatm
c              although other values are possible. In this way the
c              user should change only the mcmatm value.
c
c     mcmics = maximum allowed number of internal coordinates used
c              in Shepard interpolation. For simplicity, mcmics is
c              set to four times the value of mcmatm so the user may
c              change only the mcmatm value.
c
c     mcmicd = maximum allowed number of internal coordinates used
c              in calculating generalized distances for weights in
c              Shepard interpolation. For simplicity, mcmicd is 
c              set to four times the value of mcmatm so the user may
c              change only the mcmatm value.
c
c     mcmesp = maximum allowed number of electronic structure theory
c              Shepard points.
c
c     mcmpts = maximum number of geometries allowed for calculation
c
c
c     Note: If these parameters are modified the source code for
c           MC-TINKER must be recompiled.
c
c#######################################################################
c
      include 'mcparm.i'
c
      parameter (mcmatm = maxatm)
c
      parameter (mcmkey = maxkey)
c
c Note:  mcmics and mcmicd should be the same, e.g. (mcmics = 3*mcmatm, mcmicd = 3*mcmatm) or 
c (mcmics = 4*mcmatm, mcmicd = 4*mcmatm)
c
      parameter (mcmics = 3*mcmatm)
c
      parameter (mcmicd = 3*mcmatm)
c
      parameter (mcmesp = 20)
c
      parameter (mcmpts = 200)
c
c Note: for symmetrized PESs, the mcmicss, mcmicds, and mcmesps variables
c should be reset. (mcmicss = maximum number of the internal coordinates
c used in Shepard interpolation; (mcmicds = maximum number of the internal coordinates
c used in calculation of weights; mcmesps =  max number of the electronic structure
c Shepard points).
c
      parameter (mcmicss = 1)
      parameter (mcmicds = 1)
      parameter (mcmesps = 1)
c for symmetrized calculations uncomment these three lines:
c      parameter (mcmicss = mcmics)
c      parameter (mcmicds = mcmicd)
c      parameter (mcmesps = mcmesp)
c#######################################################################
c
c     Fortran unit number for specifying the MC-TINKER input and output
c     files (see manual for a better description of the files associated
c     with each number)
c
c     Note: The integer specification must be made before the
c           parameter declaration.
c
c     Note: If these parameters are modified the source code for
c           MC-TINKER must be recompiled.
c
c#######################################################################
c
      INTEGER FU5, FU6
      INTEGER FU83, FU84
c
      PARAMETER (FU5 = 5, FU6 = 6)
      PARAMETER (FU83 = 83, FU84 = 84)
c
c#######################################################################
