c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ################################################################
c     ##                                                            ##
c     ##  subroutine xyzatm  --  single atom internal to Cartesian  ##
c     ##                                                            ##
c     ################################################################
c
c
c     "xyzatm" computes the Cartesian coordinates of a single
c     atom from its defining internal coordinate values
c
c
      subroutine xyzatm (i,i1,bond,i2,angle1,i3,angle2,chiral)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'inform.i'
      include 'iounit.i'
      include 'math.i'
      integer i,i1,i2,i3,chiral
      real*8 bond,angle1,angle2,ang_1,ang_2
      real*8 sin_1,cos_1,sin_2,cos_2
      real*8 cosine,sine,sine2,norm,eps
      real*8 u1(3),u2(3),u3(3),u4(3),a,b,c
c
c
c     convert angles to radians, and get their sines and cosines
c
      eps = 0.00000001d0
      ang_1 = angle1 / radian
      ang_2 = angle2 / radian
      sin_1 = sin(ang_1)
      cos_1 = cos(ang_1)
      sin_2 = sin(ang_2)
      cos_2 = cos(ang_2)
c
c     first, the case where second angle is a dihedral angle
c
      if (chiral .eq. 0) then
         u1(1) = x(i2) - x(i3)
         u1(2) = y(i2) - y(i3)
         u1(3) = z(i2) - z(i3)
         norm = sqrt(u1(1)*u1(1) + u1(2)*u1(2) + u1(3)*u1(3))
         u1(1) = u1(1) / norm
         u1(2) = u1(2) / norm
         u1(3) = u1(3) / norm
         u2(1) = x(i1) - x(i2)
         u2(2) = y(i1) - y(i2)
         u2(3) = z(i1) - z(i2)
         norm = sqrt(u2(1)*u2(1) + u2(2)*u2(2) + u2(3)*u2(3))
         u2(1) = u2(1) / norm
         u2(2) = u2(2) / norm
         u2(3) = u2(3) / norm
         u3(1) = u1(2)*u2(3) - u1(3)*u2(2)
         u3(2) = u1(3)*u2(1) - u1(1)*u2(3)
         u3(3) = u1(1)*u2(2) - u1(2)*u2(1)
         cosine = u1(1)*u2(1) + u1(2)*u2(2) + u1(3)*u2(3)
         if (abs(cosine) .lt. 1.0d0) then
            sine = sqrt(1.0d0 - cosine**2)
         else
            write (iout,10)  i
   10       format (' XYZATM  --  Undefined Dihedral Angle',
     &                 ' at Atom',i6)
            sine = sqrt(cosine**2 - 1.0d0)
         end if
         u3(1) = u3(1) / sine
         u3(2) = u3(2) / sine
         u3(3) = u3(3) / sine
         u4(1) = u3(2)*u2(3) - u3(3)*u2(2)
         u4(2) = u3(3)*u2(1) - u3(1)*u2(3)
         u4(3) = u3(1)*u2(2) - u3(2)*u2(1)
         x(i) = x(i1) + bond * (-u2(1)*cos_1 + u4(1)*sin_1*cos_2
     &                                + u3(1)*sin_1*sin_2)
         y(i) = y(i1) + bond * (-u2(2)*cos_1 + u4(2)*sin_1*cos_2
     &                                 + u3(2)*sin_1*sin_2)
         z(i) = z(i1) + bond * (-u2(3)*cos_1 + u4(3)*sin_1*cos_2
     &                                 + u3(3)*sin_1*sin_2)
c
c     now, the case where second angle is a bond angle
c
      else if (abs(chiral) .eq. 1) then
         u1(1) = x(i1) - x(i3)
         u1(2) = y(i1) - y(i3)
         u1(3) = z(i1) - z(i3)
         norm = sqrt(u1(1)*u1(1) + u1(2)*u1(2) + u1(3)*u1(3))
         u1(1) = u1(1) / norm
         u1(2) = u1(2) / norm
         u1(3) = u1(3) / norm
         u2(1) = x(i2) - x(i1)
         u2(2) = y(i2) - y(i1)
         u2(3) = z(i2) - z(i1)
         norm = sqrt(u2(1)*u2(1) + u2(2)*u2(2) + u2(3)*u2(3))
         u2(1) = u2(1) / norm
         u2(2) = u2(2) / norm
         u2(3) = u2(3) / norm
         u3(1) = u1(2)*u2(3) - u1(3)*u2(2)
         u3(2) = u1(3)*u2(1) - u1(1)*u2(3)
         u3(3) = u1(1)*u2(2) - u1(2)*u2(1)
         cosine = u1(1)*u2(1) + u1(2)*u2(2) + u1(3)*u2(3)
         if (abs(cosine) .lt. 1.0d0) then
            sine2 = 1.0d0 - cosine**2
         else
            write (iout,20)  i
   20       format (' XYZATM  --  Defining Atoms Colinear',
     &                 ' at Atom',i6)
         end if
         a = (-cos_2 - cosine*cos_1) / sine2
         b = (cos_1 + cosine*cos_2) / sine2
         c = (1.0d0 + a*cos_2 - b*cos_1) / sine2
         if (c .gt. eps) then
            c = chiral * sqrt(c)
         else if (c .lt. -eps) then
            c = sqrt((a*u1(1)+b*u2(1))**2 + (a*u1(2)+b*u2(2))**2
     &                       + (a*u1(3)+b*u2(3))**2)
            a = a / c
            b = b / c
            c = 0.0d0
            if (debug) then
               write (iout,30)  i1
   30          format (' XYZATM  --  Sum of Bond Angles Too Large',
     &                    ' at Atom',i6)
            end if
         else
            c = 0.0d0
         end if
         x(i) = x(i1) + bond * (a*u1(1) + b*u2(1) + c*u3(1))
         y(i) = y(i1) + bond * (a*u1(2) + b*u2(2) + c*u3(2))
         z(i) = z(i1) + bond * (a*u1(3) + b*u2(3) + c*u3(3))
      end if
      return
      end
