c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ##############################################################
c     ##                                                          ##
c     ##  program protein  --  peptide coordinates from sequence  ##
c     ##                                                          ##
c     ##############################################################
c
c
c     "protein" builds the internal and Cartesian coordinates
c     of a polypeptide from the amino acid sequence and torsional
c     angle values for the peptide backbone and side chains
c
c
      program protein
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'files.i'
      include 'iounit.i'
      include 'titles.i'
      integer i,izmt,ixyz,iseq,natom,mode
      integer freeunit,trimtext
      character*60 seqfile,intfile,xyzfile
      logical exist
c
c
c     get the name to use for the output structure files
c
      call initial
      call nextarg (filename,exist)
      if (.not. exist) then
         write (iout,10)
   10    format (/,' Enter Name to be used for Output Files :  ',$)
         read (input,20)  filename
   20    format (a60)
      end if
      call basefile (filename)
c
c     get the title line for the output files
c
      write (iout,30)
   30 format (/,' Enter Title :  ',$)
      read (input,40)  title
   40 format (a60)
      ltitle = trimtext (title)
c
c     read the keyfile and force field parameter file
c
      call getkey
      call field
c
c     get the sequence, build a Z-matrix, convert to Cartesians
c
      call getseq
      call proteus
      call makexyz
      call connect
c
c     remove any dummy atoms from Cartesian coordinates,
c     then convert back to internal coordinates
c
      natom = n
      do i = natom, 1, -1
         if (type(i) .eq. 0)  call delete (i)
      end do
      mode = 0
      call makeint (mode)
c
c     write out a amino acid sequence file
c
      iseq = freeunit ()
      seqfile = filename(1:leng)//'.seq'
      call version (seqfile,'new')
      open (unit=iseq,file=seqfile,status='new')
      call prtseq (iseq)
      close (unit=iseq)
c
c     write out an internal coordinates file
c
      izmt = freeunit ()
      intfile = filename(1:leng)//'.int'
      call version (intfile,'new')
      open (unit=izmt,file=intfile,status='new')
      call prtint (izmt)
      close (unit=izmt)
c
c     write out a Cartesian coordinates file
c
      ixyz = freeunit ()
      xyzfile = filename(1:leng)//'.xyz'
      call version (xyzfile,'new')
      open (unit=ixyz,file=xyzfile,status='new')
      call prtxyz (ixyz)
      close (unit=ixyz)
      end
c
c
c     #############################################################
c     ##                                                         ##
c     ##  subroutine getseq  --  amino acid sequence and angles  ##
c     ##                                                         ##
c     #############################################################
c
c
c     "getseq" asks the user for the amino acid sequence
c     and torsional angle values needed to define a peptide
c
c
      subroutine getseq
      implicit none
      include 'sizes.i'
      include 'aminos.i'
      include 'iounit.i'
      include 'phipsi.i'
      include 'sequen.i'
      integer i,j,next
      integer length,trimtext
      character*1 chir(maxres)
      character*3 name
      character*80 record,string
      logical done
c
c
c     get the amino acid sequence data and dihedral angle values
c
      write (iout,10)
   10 format (/,' Enter One Residue per Line, Free Format: ',
     &           ' 1 or 3 Letter Code, then',
     &        /,' Phi/Psi/Omega (3F), Chi Angles (4F), then',
     &           ' Disulfide Partner if a',
     &        /,' CYS (I) and D/L Chirality as desired, <CR>',
     &           ' ends the Residue entry')
      i = 0
      done = .false.
      dowhile (.not. done)
         i = i + 1
         phi(i) = 0.0d0
         psi(i) = 0.0d0
         omega(i) = 0.0d0
         do j = 1, 4
            chi(j,i) = 0.0d0
         end do
         disulf(i) = 0
         chir(i) = ' '
         write (iout,20)  i
   20    format (/,' Enter Residue',i4,' :  ',$)
         read (input,30)  record
   30    format (a80)
         call upcase (record)
         next = 1
         call getword (record,name,next)
         string = record(next:80)
         read (string,*,err=40,end=40)  phi(i),psi(i),omega(i),
     &                                  (chi(j,i),j=1,4),disulf(i)
   40    continue
         call getword (string,chir(i),next)
c
c     get length of amino acid name and handle special names
c
         length = trimtext (name)
         if (name .eq. 'HIP')  name = 'HIS'
c
c     process and store the current amino acid residue type
c
         if (length .eq. 1) then
            seq(i) = amino1(maxamino)
            seqtyp(i) = maxamino
            do j = 1, maxamino
               if (name(1:1) .eq. amino1(j)) then
                  seq(i) = amino1(j)
                  seqtyp(i) = j
               end if
            end do
         else if (length .eq. 3) then
            seq(i) = amino1(maxamino)
            seqtyp(i) = maxamino
            do j = 1, maxamino
               if (name .eq. amino(j)) then
                  seq(i) = amino1(j)
                  seqtyp(i) = j
               end if
            end do
         else
            done = .true.
            nseq = i - 1
         end if
      end do
c
c     all residues are assumed to be part of a single chain
c
      nchain = 1
      ichain(1,1) = 1
      ichain(2,1) = nseq
      chnnam(1) = ' '
c
c     set default values for the phi-psi-omega-chi angles;
c     use alpha helix values if no phi-psi values were given
c
      do i = 1, nseq
         if (phi(i).eq.0.0d0 .and. psi(i).eq.0.0d0) then
            phi(i) = -63.0d0
            psi(i) = -42.0d0
         end if
         if (omega(i) .eq. 0.0d0) then
            omega(i) = 180.0d0
         end if
         if (chi(1,i) .eq. 0.0d0) then
            do j = 1, 4
               chi(j,i) = 180.0d0
               if (seq(i) .eq. 'P')  chi(j,i) = 0.0d0
               if (seq(i) .eq. 'J')  chi(j,i) = 0.0d0
            end do
            if (seq(i) .eq. 'F')  chi(2,i) = 90.0d0
            if (seq(i) .eq. 'Y')  chi(2,i) = 90.0d0
            if (seq(i) .eq. 'W')  chi(2,i) = 90.0d0
            if (seq(i) .eq. 'H')  chi(2,i) = 90.0d0
            if (seq(i) .eq. 'U')  chi(2,i) = 90.0d0
            if (seq(i) .eq. 'Z')  chi(2,i) = 90.0d0
         end if
c
c     check for any disulfide bonds
c
         if (disulf(i) .ne. 0) then
            if (seq(i) .ne. 'C') then
               write (iout,50)  i
   50          format (' GETSEQ  --  Error in Disulfide Bond',
     &                    ' at Residue',i5)
            end if
            if (i.lt.disulf(i) .and.
     &          disulf(disulf(i)).ne.i) then
               write (iout,60)  i,disulf(i)
   60          format (' GETSEQ  --  Error in Disulfide Bond',
     &                    ' at Residue',i5,' or',i5)
            end if
         end if
c
c     check the D/L chirality of the residues
c
         if (chir(i) .eq. 'D') then
            chiral(i) = -1
         else
            chiral(i) = 1
         end if
      end do
      return
      end
c
c
c     ##########################################################
c     ##                                                      ##
c     ##  subroutine zatom  --  adds single atom to Z-matrix  ##
c     ##                                                      ##
c     ##########################################################
c
c
c     "zatom" adds an atom to the end of the current Z-matrix
c     and then increments the atom counter; atom type, defining
c     atoms and internal coordinates are passed as arguments
c
c
      subroutine zatom (bionum,bond,angle,dihed,iz1,iz2,iz3,iz4)
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'fields.i'
      include 'iounit.i'
      include 'katoms.i'
      include 'zclose.i'
      include 'zcoord.i'
      integer bionum,iz1,iz2,iz3,iz4
      real*8 bond,angle,dihed
c
c
c     fill various arrays with information for this atom
c
      if (bionum .gt. 0) then
         type(n) = biotyp(bionum)
         if (type(n) .ne. 0) then
            name(n) = symbol(type(n))
         else
            name(n) = '   '
         end if
         blen(n) = bond
         alph(n) = angle
         bet(n) = dihed
         if (bet(n) .lt. -180.0d0) then
            bet(n) = bet(n) + 360.0d0
         else if (bet(n) .gt. 180.0d0) then
            bet(n) = bet(n) - 360.0d0
         end if
         iz(1,n) = iz1
         iz(2,n) = iz2
         iz(3,n) = iz3
         iz(4,n) = iz4
c
c     increment atom counter and check for too many atoms
c
         n = n + 1
         if (n .gt. maxatm) then
            write (iout,10)  maxatm
   10       format (/,' ZATOM  --  The Maximum of',i6,' Atoms',
     &                 ' has been Exceeded')
            call fatal
         end if
c
c     add an extra bond to make a ring closure
c
      else if (bionum .eq. -1) then
         nadd = nadd + 1
         iadd(1,nadd) = iz1
         iadd(2,nadd) = iz2
c
c     delete an extra bond to make separate molecules
c
      else if (bionum .eq. -2) then
         ndel = ndel + 1
         idel(1,ndel) = iz1
         idel(2,ndel) = iz2
      end if
      return
      end
c
c
c     ##########################################################
c     ##                                                      ##
c     ##  subroutine proteus  --  builds protein coordinates  ##
c     ##                                                      ##
c     ##########################################################
c
c
c     "proteus" builds up the internal coordinates for an amino
c     acid sequence from the phi, psi, omega and chi values
c
c
      subroutine proteus
      implicit none
      include 'sizes.i'
      include 'aminos.i'
      include 'atoms.i'
      include 'iounit.i'
      include 'phipsi.i'
      include 'sequen.i'
      integer i,k,ni(maxatm),cai(maxatm),ci(maxatm)
      integer ntyp(maxamino),catyp(maxamino),ctyp(maxamino)
      integer hntyp(maxamino),otyp(maxamino),hatyp(maxamino)
      character*1 answer
      logical cyclic,cblocked
      logical acetyl,formyl,methyl,amide
c
c     biopolymer atom types for amino acid backbone atoms
c
      data ntyp   /   1,   7,  15,  25,  37,  51,  61,  73,  92,
     &              103, 118, 134, 157, 174, 190, 206, 216, 228,
     &              240, 254, 267, 283, 300, 314, 321,   1 /
      data catyp  /   2,   8,  16,  26,  38,  52,  62,  74,  93,
     &              104, 119, 135, 158, 175, 191, 207, 217, 229,
     &              241, 255, 268, 284, 301, 315, 322,   2 /
      data ctyp   /   3,   9,  17,  27,  39,  53,  63,  75,  94,
     &              105, 120, 136, 159, 176, 192, 208, 218, 230,
     &              242, 256, 269, 285, 302, 316, 323,   3 /
      data hntyp  /   4,  10,  18,  28,  40,  54,  64,  76,   0,
     &              106, 121, 137, 160, 177, 193, 209, 219, 231,
     &              243, 257, 270, 286, 303, 317, 324,   4 /
      data otyp   /   5,  11,  19,  29,  41,  55,  65,  77,  95,
     &              107, 122, 138, 161, 178, 194, 210, 220, 232,
     &              244, 258, 271, 287, 304, 318, 325,   5 /
      data hatyp  /   6,  12,  20,  30,  42,  56,  66,  78,  96,
     &              108, 123, 139, 162, 179, 195, 211, 221, 233,
     &              245, 259, 272, 288, 305,   0, 326,   6 /
c
c
c     initialization of atom counter and peptide endgroups
c
      n = 1
      cyclic = .false.
      cblocked = .false.
      acetyl = .false.
      formyl = .false.
      methyl = .false.
      amide = .false.
c
c     ask about the use of various peptide endgroup options
c
      write (iout,10)
   10 format (/,' Cyclize the Polypeptide Chain [N] :  ',$)
      read (input,20)  answer
   20 format (a1)
      call upcase (answer)
      if (answer .eq. 'Y')  cyclic = .true.
      if (.not. cyclic) then
         write (iout,30)
   30    format (/,' Attach a N-terminal Endgroup [N] :  ',$)
         read (input,20)  answer
         call upcase (answer)
         if (answer .eq. 'Y') then
   40       continue
            write (iout,50)
   50       format (/,' Choose N-Acetyl (A) or N-Formyl (F)',
     &                 ' Group :  ',$)
            read (input,20)  answer
            call upcase (answer)
            if (answer .eq. 'A') then
               acetyl = .true.
            else if (answer .eq. 'F') then
               formyl = .true.
            else
               goto 40
            end if
         end if
         write (iout,60)
   60    format (/,' Attach a C-terminal Endgroup [N] :  ',$)
         read (input,20)  answer
         call upcase (answer)
         if (answer .eq. 'Y') then
            cblocked = .true.
   70       continue
            write (iout,80)
   80       format (/,' Choose N-MethylAmide (M) or Amide (A)',
     &                 ' Group :  ',$)
            read (input,20)  answer
            call upcase (answer)
            if (answer .eq. 'M') then
               methyl = .true.
            else if (answer .eq. 'A') then
               amide = .true.
            else
               goto 70
            end if
         end if
      end if
c
c     set the residue number and type for the first residue
c
      i = 1
      k = seqtyp(i)
c
c     build the first residue with an N-formyl group
c
      if (formyl) then
         call zatom (376,0.0d0,0.0d0,0.0d0,0,0,0,0)
         call zatom (377,1.22d0,0.0d0,0.0d0,1,0,0,0)
         call zatom (378,1.12d0,120.0d0,0.0d0,1,2,0,0)
         ni(i) = n
         call zatom (ntyp(k),1.34d0,125.0d0,115.0d0,1,2,3,1)
         cai(i) = n
         call zatom (catyp(k),1.46d0,121.0d0,180.0d0,ni(i),1,2,0)
         ci(i) = n
         call zatom (ctyp(k),1.51d0,111.6d0,phi(i),cai(i),ni(i),1,0)
         call zatom (otyp(k),1.22d0,122.5d0,psi(i)-180.0d0,
     &               ci(i),cai(i),ni(i),0)
         call zatom (hntyp(k),1.02d0,121.0d0,phi(i)-180.0d0,
     &               ni(i),cai(i),ci(i),0)
         call zatom (hatyp(k),1.11d0,109.5d0,107.9d0,
     &               cai(i),ni(i),ci(i),-chiral(i))
         call sidechain (i,cai(i),ni(i),ci(i))
c
c     build the first residue with an N-acetyl group
c
      else if (acetyl) then
         call zatom (372,0.0d0,0.0d0,0.0d0,0,0,0,0)
         call zatom (374,1.51d0,0.0d0,0.0d0,n-1,0,0,0)
         call zatom (375,1.22d0,122.5d0,0.0d0,2,1,0,0)
         call zatom (373,1.11d0,107.9d0,180.0d0,1,2,3,0)
         call zatom (373,1.11d0,107.9d0,109.4d0,1,2,4,1)
         call zatom (373,1.11d0,107.9d0,109.4d0,1,2,4,-1)
         ni(i) = n
         call zatom (ntyp(k),1.34d0,123.1d0,114.4d0,2,3,1,1)
         cai(i) = n
         call zatom (catyp(k),1.46d0,121.0d0,180.0d0,ni(i),2,1,0)
         ci(i) = n
         call zatom (ctyp(k),1.51d0,111.6d0,phi(i),cai(i),ni(i),2,0)
         call zatom (otyp(k),1.22d0,122.5d0,psi(i)-180.0d0,
     &               ci(i),cai(i),ni(i),0)
         call zatom (hntyp(k),1.02d0,121.0d0,phi(i)-180.0d0,
     &               ni(i),cai(i),ci(i),0)
         call zatom (hatyp(k),1.11d0,109.5d0,107.9d0,
     &               cai(i),ni(i),ci(i),-chiral(i))
         call sidechain (i,cai(i),ni(i),ci(i))
c
c     build the first residue for a cyclic peptide
c
      else if (cyclic) then
         ni(i) = n
         call zatom (ntyp(k),0.0d0,0.0d0,0.0d0,0,0,0,0)
         cai(i) = n
         call zatom (catyp(k),1.46d0,0.0d0,0.d00,ni(i),0,0,0)
         ci(i) = n
         call zatom (ctyp(k),1.51d0,110.7d0,0.0d0,cai(i),ni(i),0,0)
         call zatom (otyp(k),1.22d0,122.5d0,psi(i)-180.0d0,
     &               ci(i),cai(i),ni(i),0)
         call zatom (hntyp(k),1.02d0,121.0d0,phi(i)-180.0d0,
     &               ni(i),cai(i),ci(i),0)
         call zatom (hatyp(k),1.11d0,109.5d0,107.9d0,
     &               cai(i),ni(i),ci(i),-chiral(i))
         call sidechain (i,cai(i),ni(i),ci(i))
c
c     build the first residue as a glycine
c
      else if (amino(k) .eq. 'GLY') then
         ni(i) = n
         call zatom (339,0.0d0,0.0d0,0.0d0,0,0,0,0)
         cai(i) = n
         call zatom (340,1.50d0,0.0d0,0.0d0,ni(i),0,0,0)
         ci(i) = n
         call zatom (341,1.51d0,110.7d0,0.0d0,cai(i),ni(i),0,0)
         call zatom (343,1.22d0,122.5d0,psi(1)-180.0d0,
     &               ci(i),cai(i),ni(i),0)
         call zatom (342,1.02d0,109.5d0,phi(i),ni(i),cai(i),ci(i),0)
         call zatom (342,1.02d0,109.5d0,108.0d0,ni(i),cai(i),n-1,1)
         call zatom (342,1.02d0,109.5d0,108.0d0,ni(i),cai(i),n-2,-1)
         call zatom (344,1.11d0,109.5d0,107.9d0,
     &               cai(i),ni(i),ci(i),-chiral(i))
         call zatom (344,1.11d0,109.5d0,107.9d0,
     &               cai(i),ni(i),ci(i),chiral(i))
c
c     build the first residue as a proline
c
      else if (amino(k) .eq. 'PRO') then
         ni(i) = n
         call zatom (345,0.0d0,0.0d0,0.0d0,0,0,0,0)
         cai(i) = n
         call zatom (346,1.50d0,0.0d0,0.0d0,ni(i),0,0,0)
         ci(i) = n
         call zatom (347,1.51d0,110.7d0,0.0d0,cai(i),ni(i),0,0)
         call zatom (349,1.22d0,122.5d0,psi(1)-180.0d0,
     &               ci(i),cai(i),ni(i),0)
         call zatom (348,1.02d0,109.5d0,0.0d0,ni(i),cai(i),ci(i),0)
         call zatom (348,1.02d0,109.5d0,-120.0d0,ni(i),cai(i),ci(i),0)
         call zatom (350,1.11d0,109.5d0,107.9d0,
     &               cai(i),ni(i),ci(i),-chiral(i))
         call zatom (97,1.54d0,107.0d0,109.5d0,
     &               cai(i),ni(i),ci(i),chiral(i))
         call zatom (99,1.54d0,107.0d0,chi(1,i),n-1,cai(i),ni(i),0)
         call zatom (351,1.54d0,107.0d0,chi(2,i),n-1,n-2,cai(i),0)
         call zatom (-1,0.0d0,0.0d0,0.0d0,ni(i),n-1,0,0)
         call zatom (98,1.11d0,109.4d0,109.4d0,n-3,cai(i),n-2,1)
         call zatom (98,1.11d0,109.4d0,109.4d0,n-4,cai(i),n-3,-1)
         call zatom (100,1.11d0,109.4d0,109.4d0,n-4,n-5,n-3,1)
         call zatom (100,1.11d0,109.4d0,109.4d0,n-5,n-6,n-4,-1)
         call zatom (352,1.11d0,109.4d0,109.4d0,n-5,n-6,ni(i),1)
         call zatom (352,1.11d0,109.4d0,109.4d0,n-6,n-7,ni(i),-1)
c
c     build the first residue as a methylalanine
c
      else if (amino(k) .eq. 'AIB') then
         ni(i) = n
         call zatom (333,0.0d0,0.0d0,0.0d0,0,0,0,0)
         cai(i) = n
         call zatom (catyp(k),1.50d0,0.0d0,0.0d0,ni(i),0,0,0)
         ci(i) = n
         call zatom (335,1.51d0,110.7d0,0.0d0,cai(i),ni(i),0,0)
         call zatom (337,1.22d0,122.5d0,psi(1)-180.0d0,
     &               ci(i),cai(i),ni(i),0)
         call zatom (336,1.02d0,109.5d0,phi(i),ni(i),cai(i),ci(i),0)
         call zatom (336,1.02d0,109.5d0,108.0d0,ni(i),cai(i),n-1,1)
         call zatom (336,1.02d0,109.5d0,108.0d0,ni(i),cai(i),n-2,-1)
         call zatom (hatyp(k),1.11d0,109.5d0,107.9d0,
     &               cai(i),ni(i),ci(i),-chiral(i))
         call sidechain (i,cai(i),ni(i),ci(i))
c
c     build the first residue as a pyroglutamic acid
c
      else if (amino(k) .eq. 'PCA') then
         ni(i) = n
         call zatom (321,0.0d0,0.0d0,0.0d0,0,0,0,0)
         cai(i) = n
         call zatom (322,1.50d0,0.0d0,0.0d0,ni(i),0,0,0)
         ci(i) = n
         call zatom (323,1.51d0,110.7d0,0.0d0,cai(i),ni(i),0,0)
         call zatom (325,1.22d0,122.5d0,psi(1)-180.0d0,
     &               ci(i),cai(i),ni(i),0)
         call zatom (324,1.02d0,109.5d0,-60.0d0,ni(i),cai(i),ci(i),0)
         call zatom (326,1.11d0,109.5d0,107.9d0,
     &               cai(i),ni(i),ci(i),-chiral(i))
         call sidechain (i,cai(i),ni(i),ci(i))
c
c     build the first residue for all other standard amino acids
c
      else
         ni(i) = n
         call zatom (333,0.0d0,0.0d0,0.0d0,0,0,0,0)
         cai(i) = n
         call zatom (334,1.50d0,0.0d0,0.0d0,ni(i),0,0,0)
         ci(i) = n
         call zatom (335,1.51d0,110.7d0,0.0d0,cai(i),ni(i),0,0)
         call zatom (337,1.22d0,122.5d0,psi(1)-180.0d0,
     &               ci(i),cai(i),ni(i),0)
         call zatom (336,1.02d0,109.5d0,phi(i),ni(i),cai(i),ci(i),0)
         call zatom (336,1.02d0,109.5d0,108.0d0,ni(i),cai(i),n-1,1)
         call zatom (336,1.02d0,109.5d0,108.0d0,ni(i),cai(i),n-2,-1)
         call zatom (338,1.11d0,109.5d0,107.9d0,
     &               cai(i),ni(i),ci(i),-chiral(i))
         call sidechain (i,cai(i),ni(i),ci(i))
      end if
c
c     build atoms for residues in the middle of the chain
c
      do i = 2, nseq-1
         k = seqtyp(i)
         ni(i) = n
         call zatom (ntyp(k),1.34d0,112.7d0,psi(i-1),
     &               ci(i-1),cai(i-1),ni(i-1),0)
         cai(i) = n
         call zatom (catyp(k),1.46d0,121.0d0,omega(i-1),
     &               ni(i),ci(i-1),cai(i-1),0)
         ci(i) = n
         call zatom (ctyp(k),1.51d0,111.6d0,phi(i),
     &               cai(i),ni(i),ci(i-1),0)
         call zatom (otyp(k),1.22d0,122.5d0,psi(i)-180.0d0,
     &               ci(i),cai(i),ni(i),0)
         call zatom (hntyp(k),1.02d0,121.0d0,phi(i)-180.0d0,
     &               ni(i),cai(i),ci(i),0)
         call zatom (hatyp(k),1.11d0,109.5d0,107.9d0,
     &               cai(i),ni(i),ci(i),-chiral(i))
         call sidechain (i,cai(i),ni(i),ci(i))
      end do
c
c     set the residue number and type for the last residue
c
      i = nseq
      k = seqtyp(nseq)
c
c     build the last residue with a C-terminal amide
c
      if (amide) then
         if (nseq .gt. 1) then
            ni(i) = n
            call zatom (ntyp(k),1.34d0,112.7d0,psi(i-1),
     &                  ci(i-1),cai(i-1),ni(i-1),0)
            cai(i) = n
            call zatom (catyp(k),1.46d0,121.0d0,omega(i-1),
     &                  ni(i),ci(i-1),cai(i-1),0)
            ci(i) = n
            call zatom (ctyp(k),1.51d0,111.6d0,phi(i),
     &                  cai(i),ni(i),ci(i-1),0)
            call zatom (otyp(k),1.22d0,122.5d0,psi(i)-180.0d0,
     &                  ci(i),cai(i),ni(i),0)
            call zatom (hntyp(k),1.02d0,121.0d0,phi(i)-180.0d0,
     &                  ni(i),cai(i),ci(i),0)
            call zatom (hatyp(k),1.11d0,109.5d0,107.9d0,
     &                  cai(i),ni(i),ci(i),-chiral(i))
            call sidechain (i,cai(i),ni(i),ci(i))
         end if
         call zatom (383,1.34d0,112.7d0,psi(i),ci(i),cai(i),ni(i),0)
         call zatom (384,1.02d0,119.0d0,0.0d0,n-1,ci(i),cai(i),0)
         call zatom (384,1.02d0,119.0d0,180.0d0,n-2,ci(i),cai(i),0)
c
c     build the last residue with a C-terminal N-methylamide
c
      else if (methyl) then
         if (nseq .gt. 1) then
            ni(i) = n
            call zatom (ntyp(k),1.34d0,112.7d0,psi(i-1),
     &                  ci(i-1),cai(i-1),ni(i-1),0)
            cai(i) = n
            call zatom (catyp(k),1.46d0,121.0d0,omega(i-1),
     &                  ni(i),ci(i-1),cai(i-1),0)
            ci(i) = n
            call zatom (ctyp(k),1.51d0,111.6d0,phi(i),
     &                  cai(i),ni(i),ci(i-1),0)
            call zatom (otyp(k),1.22d0,122.5d0,psi(i)-180.0d0,
     &                  ci(i),cai(i),ni(i),0)
            call zatom (hntyp(k),1.02d0,121.0d0,phi(i)-180.0d0,
     &                  ni(i),cai(i),ci(i),0)
            call zatom (hatyp(k),1.11d0,109.5d0,107.9d0,
     &                  cai(i),ni(i),ci(i),-chiral(i))
            call sidechain (i,cai(i),ni(i),ci(i))
         end if
         call zatom (379,1.34d0,112.7d0,psi(i),ci(i),cai(i),ni(i),0)
         call zatom (381,1.46d0,121.0d0,180.0d0,n-1,ci(i),cai(i),0)
         call zatom (380,1.02d0,118.0d0,121.0d0,n-2,ci(i),n-1,1)
         call zatom (382,1.11d0,109.5d0,180.0d0,n-2,n-3,ci(i),0)
         call zatom (382,1.11d0,109.5d0,109.5d0,n-3,n-4,n-1,1)
         call zatom (382,1.11d0,109.5d0,109.5d0,n-4,n-5,n-2,-1)
c
c     build the last residue for a cyclic peptide
c
      else if (cyclic) then
         ni(i) = n
         call zatom (ntyp(k),1.34d0,112.7d0,psi(i-1),
     &               ci(i-1),cai(i-1),ni(i-1),0)
         cai(i) = n
         call zatom (catyp(k),1.46d0,121.0d0,omega(i-1),
     &               ni(i),ci(i-1),cai(i-1),0)
         ci(i) = n
         call zatom (ctyp(k),1.51d0,111.6d0,phi(i),
     &               cai(i),ni(i),ci(i-1),0)
         call zatom (-1,0.0d0,0.0d0,0.0d0,ni(1),ci(i),0,0)
         call zatom (otyp(k),1.22d0,122.5d0,psi(i)-180.0d0,
     &               ci(i),cai(i),ni(i),0)
         call zatom (hntyp(k),1.02d0,121.0d0,phi(i)-180.0d0,
     &               ni(i),cai(i),ci(i),0)
         call zatom (hatyp(k),1.11d0,109.5d0,107.9d0,
     &               cai(i),ni(i),ci(i),-chiral(i))
         call sidechain (i,cai(i),ni(i),ci(i))
c
c     build the last residue as a glycine
c
      else if (amino(k) .eq. 'GLY') then
         if (nseq .gt. 1) then
            ni(i) = n
            call zatom (359,1.34d0,112.7d0,psi(i-1),
     &                  ci(i-1),cai(i-1),ni(i-1),0)
            cai(i) = n
            call zatom (360,1.46d0,121.0d0,omega(i-1),
     &                  ni(i),ci(i-1),cai(i-1),0)
            ci(i) = n
            call zatom (361,1.51d0,111.6d0,phi(i),
     &                  cai(i),ni(i),ci(i-1),0)
            call zatom (363,1.25d0,117.0d0,psi(i)-180.0d0,
     &                  ci(i),cai(i),ni(i),0)
            call zatom (362,1.02d0,121.0d0,phi(i)-180.0d0,
     &                  ni(i),cai(i),ci(i),0)
            call zatom (364,1.11d0,109.5d0,107.9d0,
     &                  cai(i),ni(i),ci(i),-chiral(i))
            call zatom (364,1.11d0,109.5d0,107.9d0,
     &                  cai(i),ni(i),ci(i),chiral(i))
         end if
         call zatom (363,1.25d0,117.0d0,psi(i),ci(i),cai(i),ni(i),0)
c
c     build the last residue as a proline
c
      else if (amino(k) .eq. 'PRO') then
         if (nseq .gt. 1) then
            ni(i) = n
            call zatom (365,1.34d0,112.7d0,psi(i-1),
     &                  ci(i-1),cai(i-1),ni(i-1),0)
            cai(i) = n
            call zatom (366,1.46d0,121.0d0,omega(i-1),
     &                  ni(i),ci(i-1),cai(i-1),0)
            ci(i) = n
            call zatom (367,1.51d0,111.6d0,phi(i),
     &                  cai(i),ni(i),ci(i-1),0)
            call zatom (368,1.25d0,117.0d0,psi(i)-180.0d0,
     &                  ci(i),cai(i),ni(i),0)
            call zatom (369,1.11d0,109.5d0,107.9d0,
     &                  cai(i),ni(i),ci(i),-chiral(i))
            call sidechain (i,cai(i),ni(i),ci(i))
         end if
         call zatom (368,1.25d0,117.0d0,psi(i),ci(i),cai(i),ni(i),0)
c
c     build the last residue as a methylalanine
c
      else if (amino(k) .eq. 'AIB') then
         if (nseq .gt. 1) then
            ni(i) = n
            call zatom (353,1.34d0,112.7d0,psi(i-1),
     &                  ci(i-1),cai(i-1),ni(i-1),0)
            cai(i) = n
            call zatom (catyp(k),1.46d0,121.0d0,omega(i-1),
     &                  ni(i),ci(i-1),cai(i-1),0)
            ci(i) = n
            call zatom (355,1.51d0,111.6d0,phi(i),
     &                  cai(i),ni(i),ci(i-1),0)
            call zatom (357,1.25d0,117.0d0,psi(i)-180.0d0,
     &                  ci(i),cai(i),ni(i),0)
            call zatom (356,1.02d0,121.0d0,phi(i)-180.0d0,
     &                  ni(i),cai(i),ci(i),0)
            call zatom (hatyp(k),1.11d0,109.5d0,107.9d0,
     &                  cai(i),ni(i),ci(i),-chiral(i))
            call sidechain (i,cai(i),ni(i),ci(i))
         end if
         call zatom (357,1.25d0,117.0d0,psi(i),ci(i),cai(i),ni(i),0)
c
c     build the last residue for all other standard amino acids
c
      else
         if (nseq .gt. 1) then
            ni(i) = n
            call zatom (353,1.34d0,112.7d0,psi(i-1),
     &                  ci(i-1),cai(i-1),ni(i-1),0)
            cai(i) = n
            call zatom (354,1.46d0,121.0d0,omega(i-1),
     &                  ni(i),ci(i-1),cai(i-1),0)
            ci(i) = n
            call zatom (355,1.51d0,111.6d0,phi(i),
     &                  cai(i),ni(i),ci(i-1),0)
            call zatom (357,1.25d0,117.0d0,psi(i)-180.0d0,
     &                  ci(i),cai(i),ni(i),0)
            call zatom (356,1.02d0,121.0d0,phi(i)-180.0d0,
     &                  ni(i),cai(i),ci(i),0)
            call zatom (358,1.11d0,109.5d0,107.9d0,
     &                  cai(i),ni(i),ci(i),-chiral(i))
            call sidechain (i,cai(i),ni(i),ci(i))
         end if
         call zatom (357,1.25d0,117.0d0,psi(i),ci(i),cai(i),ni(i),0)
      end if
c
c     finally, set the total number of atoms in the polypeptide
c
      n = n - 1
      return
      end
c
c
c     ###########################################################
c     ##                                                       ##
c     ##  subroutine sidechain  --  builds protein sidechains  ##
c     ##                                                       ##
c     ###########################################################
c
c
c     "sidechain" builds the side chain for a single amino
c     acid residue in terms of internal coordinates
c
c        i     number of the current amino acid residue
c        cai   atom number of alpha carbon in residue i
c        ni    atom number of amide nitrogen in residue i
c        ci    atom number of carbonyl carbon in residue i
c
c
      subroutine sidechain (i,cai,ni,ci)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'phipsi.i'
      include 'sequen.i'
      integer i,cai,ni,ci
c
c
c     jump to the code for the proper sidechain
c
      goto (10,20,30,40,50,60,70,80,90,100,
     &      110,120,130,140,150,160,170,180,
     &      190,200,210,220,230,240,250,260),  seqtyp(i)
c
c     glycine residue  (GLY)
c
   10 continue
      call zatom (6,1.11d0,109.5d0,107.9d0,cai,ni,ci,chiral(i))
      return
c
c     alanine residue  (ALA)
c
   20 continue
      call zatom (13,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
      call zatom (14,1.11d0,109.4d0,chi(1,i),n-1,cai,ni,0)
      call zatom (14,1.11d0,109.4d0,109.4d0,n-2,cai,n-1,1)
      call zatom (14,1.11d0,109.4d0,109.4d0,n-3,cai,n-2,-1)
      return
c
c     valine residue  (VAL)
c
   30 continue
      call zatom (21,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
      call zatom (23,1.54d0,109.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (23,1.54d0,109.5d0,109.5d0,n-2,cai,n-1,-1)
      call zatom (22,1.11d0,109.4d0,109.4d0,n-3,cai,n-2,1)
      call zatom (24,1.11d0,109.4d0,180.0d0,n-3,n-4,cai,0)
      call zatom (24,1.11d0,109.4d0,109.4d0,n-4,n-5,n-1,1)
      call zatom (24,1.11d0,109.4d0,109.4d0,n-5,n-6,n-2,-1)
      call zatom (24,1.11d0,109.4d0,180.0d0,n-5,n-7,cai,0)
      call zatom (24,1.11d0,109.4d0,109.4d0,n-6,n-8,n-1,1)
      call zatom (24,1.11d0,109.4d0,109.4d0,n-7,n-9,n-2,-1)
      return
c
c     leucine residue  (LEU)
c
   40 continue
      call zatom (31,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
      call zatom (33,1.54d0,109.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (35,1.54d0,109.5d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (35,1.54d0,109.5d0,109.4d0,n-2,n-3,n-1,-1)
      call zatom (32,1.11d0,109.4d0,109.4d0,n-4,cai,n-3,1)
      call zatom (32,1.11d0,109.4d0,109.4d0,n-5,cai,n-4,-1)
      call zatom (34,1.11d0,109.4d0,109.4d0,n-5,n-6,n-4,1)
      call zatom (36,1.11d0,109.4d0,180.0d0,n-5,n-6,n-7,0)
      call zatom (36,1.11d0,109.4d0,109.4d0,n-6,n-7,n-1,1)
      call zatom (36,1.11d0,109.4d0,109.4d0,n-7,n-8,n-2,-1)
      call zatom (36,1.11d0,109.4d0,180.0d0,n-7,n-9,n-10,0)
      call zatom (36,1.11d0,109.4d0,109.4d0,n-8,n-10,n-1,1)
      call zatom (36,1.11d0,109.4d0,109.4d0,n-9,n-11,n-2,-1)
      return
c
c     isoleucine residue  (ILE)
c
   50 continue
      call zatom (43,1.54d0,109.5d0,109.5d0,cai,ni,ci,chiral(i))
      call zatom (45,1.54d0,109.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (47,1.54d0,109.5d0,109.5d0,n-2,cai,n-1,1)
      call zatom (49,1.54d0,109.5d0,chi(2,i),n-2,n-3,cai,0)
      call zatom (44,1.11d0,109.4d0,109.4d0,n-4,cai,n-3,-1)
      call zatom (46,1.11d0,109.4d0,109.4d0,n-4,n-5,n-2,1)
      call zatom (46,1.11d0,109.4d0,109.4d0,n-5,n-6,n-3,-1)
      call zatom (48,1.11d0,110.0d0,180.0d0,n-5,n-7,n-6,0)
      call zatom (48,1.11d0,110.0d0,109.0d0,n-6,n-8,n-1,1)
      call zatom (48,1.11d0,110.0d0,109.0d0,n-7,n-9,n-2,-1)
      call zatom (50,1.11d0,110.0d0,180.0d0,n-7,n-9,n-10,0)
      call zatom (50,1.11d0,110.0d0,109.0d0,n-8,n-10,n-1,1)
      call zatom (50,1.11d0,110.0d0,109.0d0,n-9,n-11,n-2,-1)
      return
c
c     serine residue  (SER)
c
   60 continue
      call zatom (57,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
      call zatom (59,1.41d0,107.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (58,1.11d0,109.4d0,106.7d0,n-2,cai,n-1,1)
      call zatom (58,1.11d0,109.4d0,106.7d0,n-3,cai,n-2,-1)
      call zatom (60,0.94d0,106.9d0,chi(2,i),n-3,n-4,cai,0)
      return
c
c     threonine residue  (THR)
c
   70 continue
      call zatom (67,1.54d0,109.5d0,109.5d0,cai,ni,ci,chiral(i))
      call zatom (69,1.41d0,107.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (71,1.54d0,109.5d0,107.7d0,n-2,cai,n-1,1)
      call zatom (68,1.11d0,109.4d0,106.7d0,n-3,cai,n-2,-1)
      call zatom (70,0.94d0,106.9d0,chi(2,i),n-3,n-4,cai,0)
      call zatom (72,1.11d0,110.0d0,180.0d0,n-3,n-5,cai,0)
      call zatom (72,1.11d0,110.0d0,109.0d0,n-4,n-6,n-1,1)
      call zatom (72,1.11d0,110.0d0,109.0d0,n-5,n-7,n-2,-1)
      return
c
c     cysteine residue  (CYS)
c
   80 continue
      if (disulf(i) .eq. 0) then
         call zatom (79,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
         call zatom (81,1.82d0,109.0d0,chi(1,i),n-1,cai,ni,0)
         call zatom (80,1.11d0,109.4d0,112.0d0,n-2,cai,n-1,1)
         call zatom (80,1.11d0,109.4d0,112.0d0,n-3,cai,n-2,-1)
         call zatom (82,1.34d0,96.0d0,chi(2,i),n-3,n-4,cai,0)
      else if (disulf(i) .gt. i) then
         call zatom (89,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
         call zatom (91,1.82d0,109.0d0,chi(1,i),n-1,cai,ni,0)
         call zatom (90,1.11d0,109.4d0,112.0d0,n-2,cai,n-1,1)
         call zatom (90,1.11d0,109.4d0,112.0d0,n-3,cai,n-2,-1)
         disulf(i) = n - 3
      else if (disulf(i) .lt. i) then
         call zatom (89,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
         call zatom (91,1.82d0,109.0d0,chi(1,i),n-1,cai,ni,0)
         call zatom (90,1.11d0,109.4d0,112.0d0,n-2,cai,n-1,1)
         call zatom (90,1.11d0,109.4d0,112.0d0,n-3,cai,n-2,-1)
         call zatom (-1,0.0d0,0.0d0,0.0d0,disulf(disulf(i)),n-3,0,0)
      end if
      return
c
c     proline residue  (PRO)
c
   90 continue
      call zatom (97,1.54d0,107.0d0,109.5d0,cai,ni,ci,chiral(i))
      call zatom (99,1.54d0,107.0d0,chi(1,i),n-1,cai,ni,0)
      call zatom (101,1.54d0,107.0d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (-1,0.0d0,0.0d0,0.0d0,ni,n-1,0,0)
      call zatom (98,1.11d0,109.4d0,109.4d0,n-3,cai,n-2,1)
      call zatom (98,1.11d0,109.4d0,109.4d0,n-4,cai,n-3,-1)
      call zatom (100,1.11d0,109.4d0,109.4d0,n-4,n-5,n-3,1)
      call zatom (100,1.11d0,109.4d0,109.4d0,n-5,n-6,n-4,-1)
      call zatom (102,1.11d0,109.4d0,109.4d0,n-5,n-6,ni,1)
      call zatom (102,1.11d0,109.4d0,109.4d0,n-6,n-7,ni,-1)
      return
c
c     phenylalanine residue  (PHE)
c
  100 continue
      call zatom (109,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
      call zatom (111,1.50d0,109.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (112,1.39d0,120.0d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (112,1.39d0,120.0d0,120.0d0,n-2,n-3,n-1,1)
      call zatom (114,1.39d0,120.0d0,180.0d0,n-2,n-3,n-4,0)
      call zatom (114,1.39d0,120.0d0,180.0d0,n-2,n-4,n-5,0)
      call zatom (116,1.39d0,120.0d0,0.0d0,n-2,n-4,n-5,0)
      call zatom (-1,0.0d0,0.0d0,0.0d0,n-2,n-1,0,0)
      call zatom (110,1.11d0,109.4d0,109.4d0,n-7,cai,n-6,1)
      call zatom (110,1.11d0,109.4d0,109.4d0,n-8,cai,n-7,-1)
      call zatom (113,1.10d0,120.0d0,120.0d0,n-7,n-8,n-5,1)
      call zatom (113,1.10d0,120.0d0,120.0d0,n-7,n-9,n-5,1)
      call zatom (115,1.10d0,120.0d0,120.0d0,n-7,n-9,n-5,1)
      call zatom (115,1.10d0,120.0d0,120.0d0,n-7,n-9,n-6,1)
      call zatom (117,1.10d0,120.0d0,120.0d0,n-7,n-9,n-8,1)
      return
c
c     tyrosine residue  (TYR)
c
  110 continue
      call zatom (124,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
      call zatom (126,1.50d0,109.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (127,1.39d0,120.0d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (127,1.39d0,120.0d0,120.0d0,n-2,n-3,n-1,1)
      call zatom (129,1.39d0,120.0d0,180.0d0,n-2,n-3,n-4,0)
      call zatom (129,1.39d0,120.0d0,180.0d0,n-2,n-4,n-5,0)
      call zatom (131,1.39d0,120.0d0,0.0d0,n-2,n-4,n-5,0)
      call zatom (-1,0.0d0,0.0d0,0.0d0,n-2,n-1,0,0)
      call zatom (132,1.36d0,120.0d0,120.0d0,n-1,n-2,n-3,1)
      call zatom (125,1.11d0,109.4d0,109.4d0,n-8,cai,n-7,1)
      call zatom (125,1.11d0,109.4d0,109.4d0,n-9,cai,n-8,-1)
      call zatom (128,1.10d0,120.0d0,120.0d0,n-8,n-9,n-6,1)
      call zatom (128,1.10d0,120.0d0,120.0d0,n-8,n-10,n-6,1)
      call zatom (130,1.10d0,120.0d0,120.0d0,n-8,n-10,n-6,1)
      call zatom (130,1.10d0,120.0d0,120.0d0,n-8,n-10,n-7,1)
      call zatom (133,0.97d0,108.0d0,0.0d0,n-7,n-8,n-9,0)
      return
c
c     tryptophan residue  (TRP)
c
  120 continue
      call zatom (140,1.54d0,109.5d0,109.5d0,cai,ni,ci,chiral(i))
      call zatom (142,1.50d0,109.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (143,1.35d0,126.0d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (145,1.35d0,126.0d0,108.0d0,n-2,n-3,n-1,1)
      call zatom (146,1.35d0,108.0d0,0.0d0,n-2,n-3,n-1,0)
      call zatom (148,1.35d0,108.0d0,0.0d0,n-1,n-3,n-4,0)
      call zatom (-1,0.0d0,0.0d0,0.0d0,n-3,n-1,0,0)
      call zatom (149,1.35d0,120.0d0,180.0d0,n-3,n-1,n-2,0)
      call zatom (151,1.35d0,120.0d0,0.0d0,n-2,n-4,n-1,0)
      call zatom (153,1.35d0,120.0d0,0.0d0,n-2,n-5,n-3,0)
      call zatom (155,1.35d0,120.0d0,0.0d0,n-2,n-4,n-6,0)
      call zatom (-1,0.0d0,0.0d0,0.0d0,n-2,n-1,0,0)
      call zatom (141,1.11d0,109.4d0,109.4d0,n-10,cai,n-9,1)
      call zatom (141,1.11d0,109.4d0,109.4d0,n-11,cai,n-10,-1)
      call zatom (144,1.10d0,124.0d0,124.0d0,n-10,n-11,n-8,1)
      call zatom (147,1.05d0,124.0d0,124.0d0,n-9,n-11,n-8,1)
      call zatom (150,1.10d0,120.0d0,120.0d0,n-8,n-11,n-6,1)
      call zatom (152,1.10d0,120.0d0,120.0d0,n-8,n-10,n-6,1)
      call zatom (154,1.10d0,120.0d0,120.0d0,n-8,n-10,n-7,1)
      call zatom (156,1.10d0,120.0d0,120.0d0,n-8,n-10,n-9,1)
      return
c
c     histidine (HD and HE) residue  (HIS)
c
  130 continue
      call zatom (163,1.54d0,109.5d0,109.5d0,cai,ni,ci,chiral(i))
      call zatom (165,1.50d0,109.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (166,1.35d0,126.0d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (168,1.35d0,126.0d0,108.0d0,n-2,n-3,n-1,1)
      call zatom (170,1.35d0,108.0d0,0.0d0,n-2,n-3,n-1,0)
      call zatom (172,1.35d0,108.0d0,0.0d0,n-2,n-4,n-3,0)
      call zatom (-1,0.0d0,0.0d0,.00d0,n-2,n-1,0,0)
      call zatom (164,1.11d0,109.4d0,109.4d0,n-6,cai,n-5,1)
      call zatom (164,1.11d0,109.4d0,109.4d0,n-7,cai,n-6,-1)
      call zatom (167,1.02d0,126.0d0,0.0d0,n-6,n-7,n-8,0)
      call zatom (169,1.10d0,126.0d0,126.0d0,n-6,n-8,n-4,1)
      call zatom (171,1.10d0,126.0d0,126.0d0,n-6,n-8,n-5,1)
      call zatom (173,1.02d0,126.0d0,126.0d0,n-6,n-8,n-7,1)
      return
c
c     histidine (HD only) residue  (HID)
c
  140 continue
      call zatom (180,1.54d0,109.5d0,109.5d0,cai,ni,ci,chiral(i))
      call zatom (182,1.50d0,109.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (183,1.35d0,126.0d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (185,1.35d0,126.0d0,108.0d0,n-2,n-3,n-1,1)
      call zatom (187,1.35d0,108.0d0,0.0d0,n-2,n-3,n-1,0)
      call zatom (189,1.35d0,108.0d0,0.0d0,n-2,n-4,n-3,0)
      call zatom (-1,0.0d0,0.0d0,.00d0,n-2,n-1,0,0)
      call zatom (181,1.11d0,109.4d0,109.4d0,n-6,cai,n-5,1)
      call zatom (181,1.11d0,109.4d0,109.4d0,n-7,cai,n-6,-1)
      call zatom (184,1.02d0,126.0d0,0.0d0,n-6,n-7,n-8,0)
      call zatom (186,1.10d0,126.0d0,126.0d0,n-6,n-8,n-4,1)
      call zatom (188,1.10d0,126.0d0,126.0d0,n-6,n-8,n-5,1)
      return
c
c     histidine (HE only) residue  (HIE)
c
  150 continue
      call zatom (196,1.54d0,109.5d0,109.5d0,cai,ni,ci,chiral(i))
      call zatom (198,1.50d0,109.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (199,1.35d0,126.0d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (200,1.35d0,126.0d0,108.0d0,n-2,n-3,n-1,1)
      call zatom (202,1.35d0,108.0d0,0.0d0,n-2,n-3,n-1,0)
      call zatom (204,1.35d0,108.0d0,0.0d0,n-2,n-4,n-3,0)
      call zatom (-1,0.0d0,0.0d0,.00d0,n-2,n-1,0,0)
      call zatom (197,1.11d0,109.4d0,109.4d0,n-6,cai,n-5,1)
      call zatom (197,1.11d0,109.4d0,109.4d0,n-7,cai,n-6,-1)
      call zatom (201,1.10d0,126.0d0,126.0d0,n-5,n-7,n-3,1)
      call zatom (203,1.10d0,126.0d0,126.0d0,n-5,n-7,n-4,1)
      call zatom (205,1.02d0,126.0d0,126.0d0,n-5,n-7,n-6,1)
      return
c
c     aspartic acid residue  (ASP)
c
  160 continue
      call zatom (212,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
      call zatom (214,1.51d0,107.8d0,chi(1,i),n-1,cai,ni,0)
      call zatom (215,1.25d0,117.0d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (215,1.25d0,117.0d0,126.0d0,n-2,n-3,n-1,1)
      call zatom (213,1.11d0,109.4d0,107.9d0,n-4,cai,n-3,1)
      call zatom (213,1.11d0,109.4d0,107.9d0,n-5,cai,n-4,-1)
      return
c
c     asparagine residue  (ASN)
c
  170 continue
      call zatom (222,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
      call zatom (224,1.51d0,107.8d0,chi(1,i),n-1,cai,ni,0)
      call zatom (225,1.22d0,122.5d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (226,1.34d0,112.7d0,124.0d0,n-2,n-3,n-1,1)
      call zatom (223,1.11d0,109.4d0,107.9d0,n-4,cai,n-3,1)
      call zatom (223,1.11d0,109.4d0,107.9d0,n-5,cai,n-4,-1)
      call zatom (227,1.02d0,119.0d0,0.0d0,n-3,n-5,n-6,0)
      call zatom (227,1.02d0,119.0d0,120.0d0,n-4,n-6,n-1,1)
      return
c
c     glutamic acid residue  (GLU)
c
  180 continue
      call zatom (234,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
      call zatom (236,1.54d0,109.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (238,1.51d0,107.8d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (239,1.25d0,117.0d0,chi(3,i),n-1,n-2,n-3,0)
      call zatom (239,1.25d0,117.0d0,126.0d0,n-2,n-3,n-1,1)
      call zatom (235,1.11d0,109.4d0,109.4d0,n-5,cai,n-4,1)
      call zatom (235,1.11d0,109.4d0,109.4d0,n-6,cai,n-5,-1)
      call zatom (237,1.11d0,109.4d0,107.9d0,n-6,n-7,n-5,1)
      call zatom (237,1.11d0,109.4d0,107.9d0,n-7,n-8,n-6,-1)
      return
c
c     glutamine residue  (GLN)
c
  190 continue
      call zatom (246,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
      call zatom (248,1.54d0,109.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (250,1.51d0,107.8d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (251,1.22d0,122.5d0,chi(3,i),n-1,n-2,n-3,0)
      call zatom (252,1.34d0,112.7d0,124.0d0,n-2,n-3,n-1,1)
      call zatom (247,1.11d0,109.4d0,109.4d0,n-5,cai,n-4,1)
      call zatom (247,1.11d0,109.4d0,109.4d0,n-6,cai,n-5,-1)
      call zatom (249,1.11d0,109.4d0,107.9d0,n-6,n-7,n-5,1)
      call zatom (249,1.11d0,109.4d0,107.9d0,n-7,n-8,n-6,-1)
      call zatom (253,1.02d0,119.0d0,0.0d0,n-5,n-7,n-8,0)
      call zatom (253,1.02d0,119.0d0,120.0d0,n-6,n-8,n-1,1)
      return
c
c     methionine residue  (MET)
c
  200 continue
      call zatom (260,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
      call zatom (262,1.54d0,109.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (264,1.82d0,109.0d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (265,1.82d0,96.3d0,chi(3,i),n-1,n-2,n-3,0)
      call zatom (261,1.11d0,109.4d0,109.4d0,n-4,cai,n-3,1)
      call zatom (261,1.11d0,109.4d0,109.4d0,n-5,cai,n-4,-1)
      call zatom (263,1.11d0,109.4d0,112.0d0,n-5,n-6,n-4,1)
      call zatom (263,1.11d0,109.4d0,112.0d0,n-6,n-7,n-5,-1)
      call zatom (266,1.11d0,112.0d0,180.0d0,n-5,n-6,n-7,0)
      call zatom (266,1.11d0,112.0d0,109.4d0,n-6,n-7,n-1,1)
      call zatom (266,1.11d0,112.0d0,109.4d0,n-7,n-8,n-2,-1)
      return
c
c     lysine residue  (LYS)
c
  210 continue
      call zatom (273,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
      call zatom (275,1.54d0,109.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (277,1.54d0,109.5d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (279,1.54d0,109.5d0,chi(3,i),n-1,n-2,n-3,0)
      call zatom (281,1.50d0,109.5d0,chi(4,i),n-1,n-2,n-3,0)
      call zatom (274,1.11d0,109.4d0,109.4d0,n-5,cai,n-4,1)
      call zatom (274,1.11d0,109.4d0,109.4d0,n-6,cai,n-5,-1)
      call zatom (276,1.11d0,109.4d0,109.4d0,n-6,n-7,n-5,1)
      call zatom (276,1.11d0,109.4d0,109.4d0,n-7,n-8,n-6,-1)
      call zatom (278,1.11d0,109.4d0,109.4d0,n-7,n-8,n-6,1)
      call zatom (278,1.11d0,109.4d0,109.4d0,n-8,n-9,n-7,-1)
      call zatom (280,1.11d0,109.4d0,108.8d0,n-8,n-9,n-7,1)
      call zatom (280,1.11d0,109.4d0,108.8d0,n-9,n-10,n-8,-1)
      call zatom (282,1.02d0,109.5d0,180.0d0,n-9,n-10,n-11,0)
      call zatom (282,1.02d0,109.5d0,109.5d0,n-10,n-11,n-1,1)
      call zatom (282,1.02d0,109.5d0,109.5d0,n-11,n-12,n-2,-1)
      return
c
c     arginine residue  (ARG)
c
  220 continue
      call zatom (289,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
      call zatom (291,1.54d0,109.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (293,1.54d0,109.5d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (295,1.45d0,109.5d0,chi(3,i),n-1,n-2,n-3,0)
      call zatom (297,1.35d0,120.0d0,chi(4,i),n-1,n-2,n-3,0)
      call zatom (298,1.35d0,120.0d0,180.0d0,n-1,n-2,n-3,0)
      call zatom (298,1.35d0,120.0d0,120.0d0,n-2,n-3,n-1,1)
      call zatom (290,1.11d0,109.4d0,109.4d0,n-7,cai,n-6,1)
      call zatom (290,1.11d0,109.4d0,109.4d0,n-8,cai,n-7,-1)
      call zatom (292,1.11d0,109.4d0,109.4d0,n-8,n-9,n-7,1)
      call zatom (292,1.11d0,109.4d0,109.4d0,n-9,n-10,n-8,-1)
      call zatom (294,1.11d0,109.4d0,109.4d0,n-9,n-10,n-8,1)
      call zatom (294,1.11d0,109.4d0,109.4d0,n-10,n-11,n-9,-1)
      call zatom (296,1.02d0,120.0d0,120.0d0,n-10,n-11,n-9,1)
      call zatom (299,1.02d0,120.0d0,180.0d0,n-9,n-10,n-11,0)
      call zatom (299,1.02d0,120.0d0,120.0d0,n-10,n-11,n-1,1)
      call zatom (299,1.02d0,120.0d0,180.0d0,n-10,n-12,n-13,0)
      call zatom (299,1.02d0,120.0d0,120.0d0,n-11,n-13,n-1,1)
      return
c
c     ornithine residue  (ORN)
c
  230 continue
      call zatom (306,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
      call zatom (308,1.54d0,109.5d0,chi(1,i),n-1,cai,ni,0)
      call zatom (310,1.54d0,109.5d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (312,1.50d0,109.5d0,chi(3,i),n-1,n-2,n-3,0)
      call zatom (307,1.11d0,109.4d0,109.4d0,n-4,cai,n-3,1)
      call zatom (307,1.11d0,109.4d0,109.4d0,n-5,cai,n-4,-1)
      call zatom (309,1.11d0,109.4d0,109.4d0,n-5,n-6,n-4,1)
      call zatom (309,1.11d0,109.4d0,109.4d0,n-6,n-7,n-5,-1)
      call zatom (311,1.11d0,109.4d0,109.4d0,n-6,n-7,n-5,1)
      call zatom (311,1.11d0,109.4d0,109.4d0,n-7,n-8,n-6,-1)
      call zatom (313,1.02d0,109.5d0,180.0d0,n-7,n-8,n-9,0)
      call zatom (313,1.02d0,109.5d0,109.5d0,n-8,n-9,n-1,1)
      call zatom (313,1.02d0,109.5d0,109.5d0,n-9,n-10,n-2,-1)
      return
c
c     methylalanine residue  (AIB)
c
  240 continue
      call zatom (319,1.54d0,109.5d0,107.8d0,cai,ni,ci,-chiral(i))
      call zatom (319,1.54d0,109.5d0,107.8d0,cai,ni,ci,chiral(i))
      call zatom (320,1.11d0,109.4d0,chi(1,i),n-2,cai,ni,0)
      call zatom (320,1.11d0,109.4d0,109.4d0,n-3,cai,n-2,1)
      call zatom (320,1.11d0,109.4d0,109.4d0,n-4,cai,n-3,-1)
      call zatom (320,1.11d0,109.4d0,chi(1,i),n-4,cai,ni,0)
      call zatom (320,1.11d0,109.4d0,109.4d0,n-5,cai,n-4,1)
      call zatom (320,1.11d0,109.4d0,109.4d0,n-6,cai,n-5,-1)
      return
c
c     pyroglutamic acid residue  (PCA)
c
  250 continue
      call zatom (327,1.54d0,107.0d0,109.5d0,cai,ni,ci,chiral(i))
      call zatom (329,1.54d0,107.0d0,chi(1,i),n-1,cai,ni,0)
      call zatom (331,1.54d0,107.0d0,chi(2,i),n-1,n-2,cai,0)
      call zatom (-1,0.0d0,0.0d0,0.0d0,ni,n-1,0,0)
      call zatom (332,1.22d0,126.0d0,126.0d0,n-1,ni,n-2,1)
      call zatom (328,1.11d0,109.4d0,109.4d0,n-4,cai,n-3,1)
      call zatom (328,1.11d0,109.4d0,109.4d0,n-5,cai,n-4,-1)
      call zatom (330,1.11d0,109.4d0,109.4d0,n-5,n-6,n-4,1)
      call zatom (330,1.11d0,109.4d0,109.4d0,n-6,n-7,n-5,-1)
      return
c
c     unknown residue  (UNK)
c
  260 continue
      call zatom (6,1.11d0,109.5d0,107.9d0,cai,ni,ci,chiral(i))
      return
      end
